from time import time
from urllib.parse import urlencode
from typing import TypedDict, Optional, Mapping, Union, Any

class FlayyerMeta(TypedDict, total=False):
    agent: str
    width: Union[str, int]
    height: Union[str, int]
    resolution: Union[str, int]
    id: Union[str, int]
    v: Union[str, int]

class Flayyer:
    def __init__(
        self,
        tenant: str,
        deck: str,
        template: str,
        version: Optional[int] = None,
        extension: str = "jpeg",
        variables: Optional[Mapping[Any, Any]] = None,
        meta: Optional[FlayyerMeta] = None
    ):
        self.tenant = tenant
        self.deck = deck
        self.template = template
        self.version = version
        self.extension = extension
        self.variables = variables if variables else {}
        self.meta = meta if meta else {}

    def querystring(self) -> str:
        defaults = {
            "__v": self.meta.get("v", str(int(time()))),  # This forces crawlers to refresh the image
            "__id": self.meta.get("id"),
            "_w": self.meta.get("width"),
            "_h": self.meta.get("height"),
            "_res": self.meta.get("resolution"),
            "_ua": self.meta.get("agent"),
        }
        return to_query({**defaults, **self.variables})

    def href(self) -> str:
        query = self.querystring()
        if self.version:
            return f"https://flayyer.io/v2/{self.tenant}/{self.deck}/{self.template}.{self.version}.{self.extension}?{query}"
        return f"https://flayyer.io/v2/{self.tenant}/{self.deck}/{self.template}.{self.extension}?{query}"


# From https://stackoverflow.com/a/43347067/3416691
# Alternative: https://stackoverflow.com/a/4014164/3416691
def to_query(params: Mapping[Any, Any]) -> str:
    g_encode_params = {}

    def _encode_params(params, p_key=None):
        encode_params = {}
        if params is None:
            g_encode_params[p_key] = ""  # Empty
        elif isinstance(params, dict):
            for key in params:
                encode_key = "{}[{}]".format(p_key, key)
                encode_params[encode_key] = params[key]
        elif isinstance(params, (list, tuple)):
            for offset, value in enumerate(params):
                encode_key = "{}[{}]".format(p_key, offset)
                encode_params[encode_key] = value
        elif isinstance(params, (bool)):
            g_encode_params[p_key] = str(params).lower()
        else:
            g_encode_params[p_key] = params

        for key in encode_params:
            value = encode_params[key]
            _encode_params(value, key)

    if isinstance(params, dict):
        for key in params:
            _encode_params(params[key], key)

    return urlencode(g_encode_params)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flayyer']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'flayyer',
    'version': '1.1.0',
    'description': 'FLAYYER.com helper classes and methods',
    'long_description': '# flayyer-python\n![PyPI - Version](https://img.shields.io/pypi/v/flayyer)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/flayyer)\n\nThis module is agnostic to any Python framework and has zero external dependencies.\n\nTo create a FLAYYER template please refer to: [flayyer.com](https://flayyer.com?ref=flayyer-python)\n\n## Installation\n\nInstall it with [Poetry](https://python-poetry.org/), the modern package manager.\n\n```sh\npoetry add flayyer\n```\n\nDon’t worry: if poetry is not your thing, you can also use [pip](https://pip.pypa.io/en/stable/).\n\n```sh\npip install flayyer\n```\n\n**Note:** This client requires [**Python 3.6+**](https://docs.python.org/3/whatsnew/3.6.html).\n\n## Usage\n\nAfter installing the package you can format URL as:\n\n```python\nfrom flayyer import Flayyer\n\nflayyer = Flayyer(\n    tenant="tenant",\n    deck="deck",\n    template="template",\n    variables={"title": "Hello world!"},\n)\n\n# Use this image in your <head/> tags\nurl = flayyer.href()\n# > https://flayyer.io/v2/tenant/deck/template.jpeg?__v=1596906866&title=Hello+world%21\n```\n\nVariables can be complex arrays and hashes.\n\n```python\nfrom flayyer import Flayyer, FlayyerMeta\n\nflayyer = Flayyer(\n    tenant="tenant",\n    deck="deck",\n    template="template",\n    variables={\n        "items": [\n            { "text": "Oranges", "count": 12 },\n            { "text": "Apples", "count": 14 },\n        ],\n    },\n    meta=FlayyerMeta(\n        id="slug-or-id", # To identify the resource in our analytics report\n    ),\n)\n```\n\n**IMPORTANT: variables must be serializable.**\n\nTo decode the URL for debugging purposes:\n\n```python\nfrom urllib.parse import unquote\n\nprint(unquote(url))\n# > https://flayyer.io/v2/tenant/deck/template.jpeg?title=Hello+world!&__v=123\n```\n\n## Development\n\nPrepare the local environment:\n\n```sh\npoetry install\n```\n\n```sh\npoetry shell\n```\n\nRun tests with pytest:\n\n```sh\npytest\n```\n\nDeploy with:\n\n```sh\n# Set API Token\npoetry config pypi-token.pypi pypi-TOKEN\n\npoetry version X.Y.Z\npoetry build\npoetry publish\n```\n\n## Contributing\n\nBug reports and pull requests are welcome on GitHub at https://github.com/flayyer/flayyer-python.\n\n## License\n\nThe module is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).\n\n## Acknowledgements\n\nHeads up to [**@nebil**](https://github.com/nebil) for his advice in the development of this Python module.\n',
    'author': 'Patricio López Juri',
    'author_email': 'patricio@flayyer.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://flayyer.com/',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
